<?php

interface VIEWInterface {
 function IsApplicable();
 function GetOptions();
 function GetView();
};

abstract class VIEW implements VIEWInterface {
 var $req;
 var $options;

 var $max_points;
 var $min_width, $min_height;
 var $width, $height;

 var $caches;   // CacheSet
 var $ivl;      // CacheSet Interval

 const MERGE_START = 1;
 
 const ALLOW_SAME_CHANNEL = 1;
 const FORBID_MULTIPLE_GROUPS = 2;

 const GET_RAW_DATA = 1;
 
 function __construct(REQUEST $req  = NULL, $options) {
    $this->req = $req?$req:new REQUEST();
    $this->options = $options;

    $this->object = $this->req->GetProp("view_object", false);

    if ($this->object)
        $this->max_points = $this->GetOption('max_points', 500);
    else
        $this->max_points = $this->GetOption('max_points', 5000);


    $this->aggregation = $this->GetOption('aggregation', false);
    $this->min_width = $this->GetOption('min_width', 0);
    $this->min_height = $this->GetOption('min_height', 0);

    $this->DetectImageDimensions();
    $this->CreateCacheSet();
 }

 function GetProp($prop, $default = false) {
    if (!preg_match("/^view_/", $prop))
        $prop = "view_" . $prop;
    
    return $this->req->GetProp($prop, $default);
 }

 function GetOption($opt, $default = false) {
    if (isset($this->options[$opt])) return $this->options[$opt];
    return $default;
 }

 function IsApplicable() {
    return true;
 }

 function GetOptions() {
    return array();
 }

 function DetectImageDimensions() {
    if ($this->object) {
        $width = $this->req->GetProp($this->object . "_width", $this->min_width + 20) - 20;
        if ($width < $this->min_width) $width = $this->min_width;
        $height = $width - 40;//$this->req->GetProp($this->object . "_height", $this->min_height);
    } else {
        $width = $this->req->GetProp("page_width", $this->min_width + 5) - 5;
        $height = $this->req->GetProp("page_height", $this->min_height);
        if ($width < $this->min_width) $width = $this->min_width;
        if ($height < $this->min_height) $height = $this->min_height;
    }
    
    $this->width = $width;
    $this->height = $height;
 }

 function CreateCacheSet() {
    if (!$this->caches) {
        $req = $this->req->CreateDataRequest();

        $rdr = $req->CreateReader();
        $group = $rdr->CreateGroup();
        $caches = $rdr->CreateCacheSet($group, $mask);
        $iv = $caches->CreateInterval($req, true);
        
        $this->caches = $caches;
        $this->ivl = $iv;
    }
    
 }

 function ParseDataOptions($params, $ignore_missing = false) {
    $ids = array();    

    foreach ($params as $param) {
        $x = $this->GetProp($param, false);
        if ($x) {
            list($x_gid, $x_id) = explode(":", $x);
        } else if (!$ignore_missing) {
            throw new ADEIException(translate("Parameter $param is not set"));
        } 
        
        if (!is_array($ids[$x_gid]))
            $ids[$x_gid] = array();
        array_push($ids[$x_gid], $x_id);
    }
    
    return $ids;
 }

 function GetDataVariants($params, $flags = 0) {
    $this->CreateCacheSet();

    if (!is_array($params)) 
        $params = array($params);
        
    $result = array();
    for ($i = 0; $i < sizeof($params); $i++) {    
        $result[$i] = array();
    }
    
    $gid = 0;
    $fixed_gid = false;
    foreach ($this->caches as $key => $cachewrap) {
        for ($i = 0; $i < sizeof($params); $i++) {    
	    array_push($result[$i], array("label" => $cachewrap->GetGroupTitle(), "disabled" => 1));
        }

        $id = 0;
	$list = $cachewrap->GetItemList();
	$values = array();
	foreach ($list as $id => $info) {
	    $title = $info['name'];
            for ($i = 0; $i < sizeof($params); $i++) {    
                $selected_chan = false;

                $idx = array_search("$gid:$id", $values);
                if ($idx === false) {
                        // free we can assign
                    if (!isset($values[$i])) {
                        $setval = $this->GetProp($params[$i], false);
                        if ($setval) {
                                // What if it invalid value? It will be just left unselected.
                            $values[$i] = $setval;
                        } else {
                            $values[$i] ="$gid:$id";
                        }
                    }
                } else if ($idx < $i) {
                        // Already used before
                    if (($flags&VIEW::ALLOW_SAME_CHANNEL) == 0)
                        continue;
                } else {
                        // Even if it set for later input, we can use it here and override...
                        // If it is invalid, the earlier is also set to something valid/invalid anyway...
                }

                if ($i) {
                    if ($fixed_gid !== false) {
                        if ($gid != $fixed_gid)
                            continue;
                    }
                } else {
                    if (($flags&VIEW::FORBID_MULTIPLE_GROUPS)&&(isset($values[$i]))) {
                        list($fixed_gid, $tmp) = explode(":", $values[$i]);
                    }
                }
                
	        array_push($result[$i], array("value" => "$gid:$id", "label" => "  $title"));
	    }
	    $id++;
	}
	$gid++;
    }

    return $result;
 }

 function GetDataOptions($params, $flags = 0) {
    $result = $this->GetDataVariants($params, $flags);

    if (!is_array($params)) 
        $params = array($params);
    
    $res = array();    
    for ($i = 0; $i < sizeof($params); $i++) {
        array_push($res, array("select" => array("label" => $params[$i], "id" => $params[$i], "options" => $result[$i])));
        array_push($res, array("xml"=>"<br/>"));
    }
    
    return $res;
 }

 function GetData($params, INTERVAL $iv = NULL, $flags = 0) {
    $this->CreateCacheSet();
    if (!$iv) $iv = $this->ivl;

    if (!is_array($params)) 
        $params = array($params);

    $ids = $this->ParseDataOptions($params);

    $window_size = $iv->GetWindowSize();
    $window_start = $iv->GetWindowStart();
    $window_end = $iv->GetWindowEnd();

    $rescfg = array(
        'limit' => $this->max_points,
    );

    $gid = 0;
    $gpos = 0;
    $data = array();
    foreach ($this->caches as $key => $cachewrap) {
        if (!isset($ids[$gid])) {
            $gid++;
	    continue;
        }



        if ($flags&VIEW::GET_RAW_DATA) {
	    $points = $cachewrap->GetAllPoints($this->ivl, $this->max_points);
            $operation_info = array("resolution" => 0);
        } else {
            $resolution = $cachewrap->GetResolution();
            if ($this->aggregation === false) {	    
                $r = $resolution->Get($iv, $this->width);
                $size = $resolution->GetWindowSize($r);
            
                if (($size > 0)&&(($window_size / $size) > $this->max_points)) {
                    $new_r = $resolution->Larger($r);
                    if ($new_r !== false) $r = $new_r;
                }
            } else {
                $r = $resolution->GetByWindowSize($this->aggregation);
            }
            $rescfg['resolution'] = $r;
            $points = $cachewrap->GetIntervals($iv, $rescfg, CACHE::TRUNCATE_INTERVALS);
            $operation_info = $points->GetOperationInfo();
        }
        

        foreach ($ids[$gid] as $i => $id) {
            $info[$i] = $operation_info;
        }

        foreach($points as $t => $v) {
            foreach ($ids[$gid] as $i => $id) {
                if ($flags&VIEW::GET_RAW_DATA) $val = $v[$id];
                else $val = $v['mean'.$id];

                if (is_numeric($val)) {
                    if (!is_array($data[$t])) $data[$t] = array();
                    $data[$t][$gpos + $i] = $val;
                }
            }
        }

        $gid++;
        $gpos += sizeof($ids[$i]); 
    }
    
    return array("info" => $info, "data" => $data);
 }

 function MergeViewInfo(&$view, $info, $flags = 0) {
    $done = false;
    foreach ($view as &$v) {
        if (isset($v['info'])) {
            if ($flags&VIEW::MERGE_START)
                $v['info'] = array_merge($info, $v['info']);
            else
                $v['info'] = array_merge($v['info'], $info);
            
            $done = true;
            break;
        }
    }
    
    if (!$done) {
        array_push($view, array("info" => $info));
    }
    
    return $view;
 }
 
};


?>