<?php
    $TITLE = "ADEI Log Analyzer";
    $MODULES = array("config", "download", "graph", "wiki");

    $LOG_AGGREGATES = array(
	'servers' => array(
	    'var' => '$log["server"]',
	    're_filters' => array('$log["source"]' => '/^SERVICE/'),
	    'list' => array('fpd', 'orca', 'mos', 'mos0', 'csmos', 'hv', 'temp0', 'magnetometer0', 'precmm0', 'mobs', 'aircoils', 'csms', 'cscps', 'cscryo', 'cswgts', 'csrear', 'lara', 'cstloops', 'virtual', '*'),
	),
	'caching' => array(
	    'var' => '$log["server"]',
	    're_filters' => array(),
	    'list' => array('fpd', 'orca', 'mos', 'mos0', 'csmos', 'hv', 'temp0', 'magnetometer0', 'precmm0', 'mobs', 'aircoils', 'csms', 'cscps', 'cscryo', 'cswgts', 'csrear', 'lara', 'cstloops', '*'),
	),
	'services' => array(
	    'var' => '$log["source"]',
	    're_filters' => array('$log["source"]' => '/^SERVICE/'),
	    'list' => array('SERVICE(list)', 'SERVICE(info)', 'SERVICE(srctree)', 'SERVICE(menu)', 'SERVICE(update)', 'SERVICE(legend)', 'SERVICE(getdata)', 'SERVICE(getimage)', 'SERVICE(control)',  'SERVICE(wiki)', 'SERVICE(download)', '*'),
	),
	'jobs' => array(
	    'var' => '$log["source"]',
	    're_filters' => array('$log["source"]' => '/^SYSTEM/'),
	    'list' => array('SYSTEM(cache)', 'SYSTEM(downloads_check)', '*'),
	),
	'clients' => array(
	    'var' => '$log["client"]',
	    're_filters' => array('$log["source"]' => '/^SERVICE/'),
	    'list' => array('141.52.64.189', '141.52.12.122', '141.52.12.41', '141.52.12.46', '141.52.12.56', '141.52.12.78', '141.52.16.94', '141.52.51.177', '*')
	)
    );

    $READER_DB_LOGS = array (
	"logs" => array (
	    "title" => _("LOG Analyzer"),
	    "reader" => "LOGReader",
	    "database" => array("stats", "caching", "servers", "services", "jobs", "clients")
	)
    );

    if (is_array($READER_DB)) $READER_DB = array_merge($READER_DB_LOGS, $READER_DB);
    else $READER_DB = $READER_DB_LOGS;

    $COMMON_OPTS = array(
	'slow_threshold' => 2, /* s */
	'stat_window' => 60, /* s */
    );
    
    $OPTIONS = array_merge ($OPTIONS, array(
	"logs" => array(
	    "axis" => array(
		"/latenc/" => "s",
		"/(setup|source|service)$/" => "id",
		"*" =>"#"
	    ),
	    "date_limit" => "2016-10-20 00:00:00",
	    "maximal_allowed_gap" => 60,
            "min_resolution" => 3600,
            "ignore_subseconds" => true,
            "optimize_empty_cache" => true,
            "use_cache_timewindow" => true,
	),
	"logs__stats" => array(
	    'groups' => array(
/*		'requests' => array(
		    'name' => 'Requests',
		    'items' => array(
			'latency', 'errors' // 'setup', 'source', 'service'
		    )
		),*/
		'stat' => array(
		    'name' => 'Request Statistics',
		    'items' => array(
			'requests', 'errors', 'slow_requests', 'avg_latency', 'max_latency', 'avg_parallel_requests', 'max_parallel_requests', 
		    )
		),
/*		'slow' => array(
		    'name' => 'Slow Request Statistics',
		    'items' => array(
			'avg_latency', 'avg_parallel_requests'
		    )
		),*/
/*		'sysrequests' => array(
		    'name' => 'Batch Jobs',
		    'logs' => 'requests',
		    'items' => array(
			'latency', 'errors'
		    )
		),*/
		'sysstat' => array(
		    'name' => 'Batch Job Statistics',
		    'logs' => 'stat',
		    'items' => array(
			'requests', 'slow_requests', 'avg_latency', 'max_latency', 'avg_parallel_requests', 'max_parallel_requests', 
		    )
		),
/*		'crequests' => array(
		    'name' => 'Control Set/Send Requests',
		    'logs' => 'requests',
		    'filter' => array(
			'source' => 'SERVICE(control)'
		    ),
		    'items' => array(
			'latency', 'control_lock_latency', 'zeus_dbrd_latency', 'zeus_conn_latency', 'zeus_resp_latency'
		    )
		),
		'cstat' => array(
		    'name' => 'Control Set/Send Statistics',
		    'logs' => 'stat',
		    'filter' => array(
			'source' => 'SERVICE(control)'
		    ),
		    'items' => array(
			'requests', 'slow_requests', 'avg_latency', 'max_latency', 'avg_parallel_requests', 'max_parallel_requests', 
		    )
		),*/
	    )
	),
	"logs__stats__requests" => array(
            "min_resolution" => 60,
            "ignore_subseconds" => false,
	    'opts' => array_merge($COMMON_OPTS, array(
		'report' => 'requests',
		're_filters' => array(
		    '$log["source"]' => '/^SERVICE/'
		)
	    ))
	),
	"logs__stats__stat" => array(
	    'opts' => array_merge($COMMON_OPTS, array(
		're_filters' => array(
		    '$log["source"]' => '/^SERVICE/'
		)
	    ))
	),
	"logs__stats__sysrequests" => array(
            "min_resolution" => 60,
            "ignore_subseconds" => false,
	    'opts' => array_merge($COMMON_OPTS, array(
		'report' => 'requests',
		're_filters' => array(
		    '$log["source"]' => '/^SYSTEM/'
		)
	    ))
	),
	"logs__stats__sysstat" => array(
	    'opts' => array_merge($COMMON_OPTS, array(
		're_filters' => array(
		    '$log["source"]' => '/^SYSTEM/'
		)
	    ))
	),
	"logs__stats__crequests" => array(
            "min_resolution" => 60,
            "ignore_subseconds" => false,
	    'opts' => array_merge($COMMON_OPTS, array(
		'report' => 'requests',
		're_filters' => array(
		    '$log["target"]' => '/^(set|send)/'
		)
	    ))
	),
	"logs__stats__cstat" => array(
	    'opts' => array_merge($COMMON_OPTS, array(
		're_filters' => array(
		    '$log["target"]' => '/^(set|send)/'
		)
	    ))
	)
    ));

    foreach ($LOG_AGGREGATES as $db => &$agg) {
	$var = $agg['var'];
	$list = $agg['list'];
	$re_filters = $agg['re_filters'];

	$OPTIONS["logs__$db"] = array(
	    'opts' => array_merge($COMMON_OPTS, array(
		'aggregates' => array(
		    $db => $var
		),
		're_filters' => $re_filters,
	    )),
	    'groups' => array(
		'avg_latencies' => array(
		    'name' => 'Average Latencies',
		    'items' => $list
		),
		'max_latencies' => array(
		    'name' => 'Max Latencies',
		    'items' => $list
		),
		'total_requests' => array(
		    'name' => 'Number of requests',
		    'items' => $list
		),
		'errors' => array(
		    'name' => 'Errors',
		    'items' => $list
		),
		'slow_requests' => array(
		    'name' => 'Number of slow requests',
		    'items' => $list
		),
		'slow_avg_latencies' => array(
		    'name' => 'Average latencies of slow requests',
		    'items' => $list
		),
		'avg_parallel_requests' => array(
		    'name' => 'Average number of parallel requests of the specified type',
		    'items' => $list
		),
		'max_parallel_requests' => array(
		    'name' => 'Maximal number of parallel requests of the specified type',
		    'items' => $list
		),
	    )
	);
    }

    foreach ($OPTIONS["logs__caching"]['groups'] as &$cg) {
	$cg['filter'] = array(
	    'source' => 'SYSTEM(cache)'
	);
    }
?>